/*
 * Copyright (c) 2018 ARM Limited.
 *
 * SPDX-License-Identifier: MIT
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
#include "arm_compute/core/NEON/kernels/convolution/depthwise/impl_fp32_fp32.hpp"

namespace depthwise
{
using Conv = DepthwiseConvolution<3, 3, 3, 3, 1, 1, float, float>;
using ConvImpl = DepthwiseConvolutionImpl<3, 3, 3, 3, 1, 1, float, float>;

template <>
const Conv::TileFn Conv::tile_fns
  [max_in_pad_top]
  [max_in_pad_left]
  [max_in_pad_bottom]
  [max_in_pad_right]
  [max_out_pad_bottom]
  [max_out_pad_right] = {
  {  // Input pad top = 0
    {  // Input pad left = 0
      {  // Input pad bottom = 0
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 0, 0, 0, 0>,
            Conv::template process_tile<0, 0, 0, 0, 0, 1>,
            Conv::template process_tile<0, 0, 0, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 0, 0, 1, 0>,
            Conv::template process_tile<0, 0, 0, 0, 1, 1>,
            Conv::template process_tile<0, 0, 0, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 0, 0, 2, 0>,
            Conv::template process_tile<0, 0, 0, 0, 2, 1>,
            Conv::template process_tile<0, 0, 0, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 0, 1, 0, 0>,
            Conv::template process_tile<0, 0, 0, 1, 0, 1>,
            Conv::template process_tile<0, 0, 0, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 0, 1, 1, 0>,
            Conv::template process_tile<0, 0, 0, 1, 1, 1>,
            Conv::template process_tile<0, 0, 0, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 0, 1, 2, 0>,
            Conv::template process_tile<0, 0, 0, 1, 2, 1>,
            Conv::template process_tile<0, 0, 0, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 0, 2, 0, 0>,
            Conv::template process_tile<0, 0, 0, 2, 0, 1>,
            Conv::template process_tile<0, 0, 0, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 0, 2, 1, 0>,
            Conv::template process_tile<0, 0, 0, 2, 1, 1>,
            Conv::template process_tile<0, 0, 0, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 0, 2, 2, 0>,
            Conv::template process_tile<0, 0, 0, 2, 2, 1>,
            Conv::template process_tile<0, 0, 0, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 0, 3, 0, 0>,
            Conv::template process_tile<0, 0, 0, 3, 0, 1>,
            Conv::template process_tile<0, 0, 0, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 0, 3, 1, 0>,
            Conv::template process_tile<0, 0, 0, 3, 1, 1>,
            Conv::template process_tile<0, 0, 0, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 0, 3, 2, 0>,
            Conv::template process_tile<0, 0, 0, 3, 2, 1>,
            Conv::template process_tile<0, 0, 0, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 0
      {  // Input pad bottom = 1
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 1, 0, 0, 0>,
            Conv::template process_tile<0, 0, 1, 0, 0, 1>,
            Conv::template process_tile<0, 0, 1, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 1, 0, 1, 0>,
            Conv::template process_tile<0, 0, 1, 0, 1, 1>,
            Conv::template process_tile<0, 0, 1, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 1, 0, 2, 0>,
            Conv::template process_tile<0, 0, 1, 0, 2, 1>,
            Conv::template process_tile<0, 0, 1, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 1, 1, 0, 0>,
            Conv::template process_tile<0, 0, 1, 1, 0, 1>,
            Conv::template process_tile<0, 0, 1, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 1, 1, 1, 0>,
            Conv::template process_tile<0, 0, 1, 1, 1, 1>,
            Conv::template process_tile<0, 0, 1, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 1, 1, 2, 0>,
            Conv::template process_tile<0, 0, 1, 1, 2, 1>,
            Conv::template process_tile<0, 0, 1, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 1, 2, 0, 0>,
            Conv::template process_tile<0, 0, 1, 2, 0, 1>,
            Conv::template process_tile<0, 0, 1, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 1, 2, 1, 0>,
            Conv::template process_tile<0, 0, 1, 2, 1, 1>,
            Conv::template process_tile<0, 0, 1, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 1, 2, 2, 0>,
            Conv::template process_tile<0, 0, 1, 2, 2, 1>,
            Conv::template process_tile<0, 0, 1, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 1, 3, 0, 0>,
            Conv::template process_tile<0, 0, 1, 3, 0, 1>,
            Conv::template process_tile<0, 0, 1, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 1, 3, 1, 0>,
            Conv::template process_tile<0, 0, 1, 3, 1, 1>,
            Conv::template process_tile<0, 0, 1, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 1, 3, 2, 0>,
            Conv::template process_tile<0, 0, 1, 3, 2, 1>,
            Conv::template process_tile<0, 0, 1, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 1
      {  // Input pad bottom = 2
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 2, 0, 0, 0>,
            Conv::template process_tile<0, 0, 2, 0, 0, 1>,
            Conv::template process_tile<0, 0, 2, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 2, 0, 1, 0>,
            Conv::template process_tile<0, 0, 2, 0, 1, 1>,
            Conv::template process_tile<0, 0, 2, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 2, 0, 2, 0>,
            Conv::template process_tile<0, 0, 2, 0, 2, 1>,
            Conv::template process_tile<0, 0, 2, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 2, 1, 0, 0>,
            Conv::template process_tile<0, 0, 2, 1, 0, 1>,
            Conv::template process_tile<0, 0, 2, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 2, 1, 1, 0>,
            Conv::template process_tile<0, 0, 2, 1, 1, 1>,
            Conv::template process_tile<0, 0, 2, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 2, 1, 2, 0>,
            Conv::template process_tile<0, 0, 2, 1, 2, 1>,
            Conv::template process_tile<0, 0, 2, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 2, 2, 0, 0>,
            Conv::template process_tile<0, 0, 2, 2, 0, 1>,
            Conv::template process_tile<0, 0, 2, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 2, 2, 1, 0>,
            Conv::template process_tile<0, 0, 2, 2, 1, 1>,
            Conv::template process_tile<0, 0, 2, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 2, 2, 2, 0>,
            Conv::template process_tile<0, 0, 2, 2, 2, 1>,
            Conv::template process_tile<0, 0, 2, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 2, 3, 0, 0>,
            Conv::template process_tile<0, 0, 2, 3, 0, 1>,
            Conv::template process_tile<0, 0, 2, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 2, 3, 1, 0>,
            Conv::template process_tile<0, 0, 2, 3, 1, 1>,
            Conv::template process_tile<0, 0, 2, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 2, 3, 2, 0>,
            Conv::template process_tile<0, 0, 2, 3, 2, 1>,
            Conv::template process_tile<0, 0, 2, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 2
      {  // Input pad bottom = 3
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 3, 0, 0, 0>,
            Conv::template process_tile<0, 0, 3, 0, 0, 1>,
            Conv::template process_tile<0, 0, 3, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 3, 0, 1, 0>,
            Conv::template process_tile<0, 0, 3, 0, 1, 1>,
            Conv::template process_tile<0, 0, 3, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 3, 0, 2, 0>,
            Conv::template process_tile<0, 0, 3, 0, 2, 1>,
            Conv::template process_tile<0, 0, 3, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 3, 1, 0, 0>,
            Conv::template process_tile<0, 0, 3, 1, 0, 1>,
            Conv::template process_tile<0, 0, 3, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 3, 1, 1, 0>,
            Conv::template process_tile<0, 0, 3, 1, 1, 1>,
            Conv::template process_tile<0, 0, 3, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 3, 1, 2, 0>,
            Conv::template process_tile<0, 0, 3, 1, 2, 1>,
            Conv::template process_tile<0, 0, 3, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 3, 2, 0, 0>,
            Conv::template process_tile<0, 0, 3, 2, 0, 1>,
            Conv::template process_tile<0, 0, 3, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 3, 2, 1, 0>,
            Conv::template process_tile<0, 0, 3, 2, 1, 1>,
            Conv::template process_tile<0, 0, 3, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 3, 2, 2, 0>,
            Conv::template process_tile<0, 0, 3, 2, 2, 1>,
            Conv::template process_tile<0, 0, 3, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 0, 3, 3, 0, 0>,
            Conv::template process_tile<0, 0, 3, 3, 0, 1>,
            Conv::template process_tile<0, 0, 3, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 0, 3, 3, 1, 0>,
            Conv::template process_tile<0, 0, 3, 3, 1, 1>,
            Conv::template process_tile<0, 0, 3, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 0, 3, 3, 2, 0>,
            Conv::template process_tile<0, 0, 3, 3, 2, 1>,
            Conv::template process_tile<0, 0, 3, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 3
    },  // Input pad left = 0
    {  // Input pad left = 1
      {  // Input pad bottom = 0
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 0, 0, 0, 0>,
            Conv::template process_tile<0, 1, 0, 0, 0, 1>,
            Conv::template process_tile<0, 1, 0, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 0, 0, 1, 0>,
            Conv::template process_tile<0, 1, 0, 0, 1, 1>,
            Conv::template process_tile<0, 1, 0, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 0, 0, 2, 0>,
            Conv::template process_tile<0, 1, 0, 0, 2, 1>,
            Conv::template process_tile<0, 1, 0, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 0, 1, 0, 0>,
            Conv::template process_tile<0, 1, 0, 1, 0, 1>,
            Conv::template process_tile<0, 1, 0, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 0, 1, 1, 0>,
            Conv::template process_tile<0, 1, 0, 1, 1, 1>,
            Conv::template process_tile<0, 1, 0, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 0, 1, 2, 0>,
            Conv::template process_tile<0, 1, 0, 1, 2, 1>,
            Conv::template process_tile<0, 1, 0, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 0, 2, 0, 0>,
            Conv::template process_tile<0, 1, 0, 2, 0, 1>,
            Conv::template process_tile<0, 1, 0, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 0, 2, 1, 0>,
            Conv::template process_tile<0, 1, 0, 2, 1, 1>,
            Conv::template process_tile<0, 1, 0, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 0, 2, 2, 0>,
            Conv::template process_tile<0, 1, 0, 2, 2, 1>,
            Conv::template process_tile<0, 1, 0, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 0, 3, 0, 0>,
            Conv::template process_tile<0, 1, 0, 3, 0, 1>,
            Conv::template process_tile<0, 1, 0, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 0, 3, 1, 0>,
            Conv::template process_tile<0, 1, 0, 3, 1, 1>,
            Conv::template process_tile<0, 1, 0, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 0, 3, 2, 0>,
            Conv::template process_tile<0, 1, 0, 3, 2, 1>,
            Conv::template process_tile<0, 1, 0, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 0
      {  // Input pad bottom = 1
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 1, 0, 0, 0>,
            Conv::template process_tile<0, 1, 1, 0, 0, 1>,
            Conv::template process_tile<0, 1, 1, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 1, 0, 1, 0>,
            Conv::template process_tile<0, 1, 1, 0, 1, 1>,
            Conv::template process_tile<0, 1, 1, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 1, 0, 2, 0>,
            Conv::template process_tile<0, 1, 1, 0, 2, 1>,
            Conv::template process_tile<0, 1, 1, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 1, 1, 0, 0>,
            Conv::template process_tile<0, 1, 1, 1, 0, 1>,
            Conv::template process_tile<0, 1, 1, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 1, 1, 1, 0>,
            Conv::template process_tile<0, 1, 1, 1, 1, 1>,
            Conv::template process_tile<0, 1, 1, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 1, 1, 2, 0>,
            Conv::template process_tile<0, 1, 1, 1, 2, 1>,
            Conv::template process_tile<0, 1, 1, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 1, 2, 0, 0>,
            Conv::template process_tile<0, 1, 1, 2, 0, 1>,
            Conv::template process_tile<0, 1, 1, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 1, 2, 1, 0>,
            Conv::template process_tile<0, 1, 1, 2, 1, 1>,
            Conv::template process_tile<0, 1, 1, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 1, 2, 2, 0>,
            Conv::template process_tile<0, 1, 1, 2, 2, 1>,
            Conv::template process_tile<0, 1, 1, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 1, 3, 0, 0>,
            Conv::template process_tile<0, 1, 1, 3, 0, 1>,
            Conv::template process_tile<0, 1, 1, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 1, 3, 1, 0>,
            Conv::template process_tile<0, 1, 1, 3, 1, 1>,
            Conv::template process_tile<0, 1, 1, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 1, 3, 2, 0>,
            Conv::template process_tile<0, 1, 1, 3, 2, 1>,
            Conv::template process_tile<0, 1, 1, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 1
      {  // Input pad bottom = 2
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 2, 0, 0, 0>,
            Conv::template process_tile<0, 1, 2, 0, 0, 1>,
            Conv::template process_tile<0, 1, 2, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 2, 0, 1, 0>,
            Conv::template process_tile<0, 1, 2, 0, 1, 1>,
            Conv::template process_tile<0, 1, 2, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 2, 0, 2, 0>,
            Conv::template process_tile<0, 1, 2, 0, 2, 1>,
            Conv::template process_tile<0, 1, 2, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 2, 1, 0, 0>,
            Conv::template process_tile<0, 1, 2, 1, 0, 1>,
            Conv::template process_tile<0, 1, 2, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 2, 1, 1, 0>,
            Conv::template process_tile<0, 1, 2, 1, 1, 1>,
            Conv::template process_tile<0, 1, 2, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 2, 1, 2, 0>,
            Conv::template process_tile<0, 1, 2, 1, 2, 1>,
            Conv::template process_tile<0, 1, 2, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 2, 2, 0, 0>,
            Conv::template process_tile<0, 1, 2, 2, 0, 1>,
            Conv::template process_tile<0, 1, 2, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 2, 2, 1, 0>,
            Conv::template process_tile<0, 1, 2, 2, 1, 1>,
            Conv::template process_tile<0, 1, 2, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 2, 2, 2, 0>,
            Conv::template process_tile<0, 1, 2, 2, 2, 1>,
            Conv::template process_tile<0, 1, 2, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 2, 3, 0, 0>,
            Conv::template process_tile<0, 1, 2, 3, 0, 1>,
            Conv::template process_tile<0, 1, 2, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 2, 3, 1, 0>,
            Conv::template process_tile<0, 1, 2, 3, 1, 1>,
            Conv::template process_tile<0, 1, 2, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 2, 3, 2, 0>,
            Conv::template process_tile<0, 1, 2, 3, 2, 1>,
            Conv::template process_tile<0, 1, 2, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 2
      {  // Input pad bottom = 3
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 3, 0, 0, 0>,
            Conv::template process_tile<0, 1, 3, 0, 0, 1>,
            Conv::template process_tile<0, 1, 3, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 3, 0, 1, 0>,
            Conv::template process_tile<0, 1, 3, 0, 1, 1>,
            Conv::template process_tile<0, 1, 3, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 3, 0, 2, 0>,
            Conv::template process_tile<0, 1, 3, 0, 2, 1>,
            Conv::template process_tile<0, 1, 3, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 3, 1, 0, 0>,
            Conv::template process_tile<0, 1, 3, 1, 0, 1>,
            Conv::template process_tile<0, 1, 3, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 3, 1, 1, 0>,
            Conv::template process_tile<0, 1, 3, 1, 1, 1>,
            Conv::template process_tile<0, 1, 3, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 3, 1, 2, 0>,
            Conv::template process_tile<0, 1, 3, 1, 2, 1>,
            Conv::template process_tile<0, 1, 3, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 3, 2, 0, 0>,
            Conv::template process_tile<0, 1, 3, 2, 0, 1>,
            Conv::template process_tile<0, 1, 3, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 3, 2, 1, 0>,
            Conv::template process_tile<0, 1, 3, 2, 1, 1>,
            Conv::template process_tile<0, 1, 3, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 3, 2, 2, 0>,
            Conv::template process_tile<0, 1, 3, 2, 2, 1>,
            Conv::template process_tile<0, 1, 3, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<0, 1, 3, 3, 0, 0>,
            Conv::template process_tile<0, 1, 3, 3, 0, 1>,
            Conv::template process_tile<0, 1, 3, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<0, 1, 3, 3, 1, 0>,
            Conv::template process_tile<0, 1, 3, 3, 1, 1>,
            Conv::template process_tile<0, 1, 3, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<0, 1, 3, 3, 2, 0>,
            Conv::template process_tile<0, 1, 3, 3, 2, 1>,
            Conv::template process_tile<0, 1, 3, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 3
    },  // Input pad left = 1
  },  // Input pad top = 0
  {  // Input pad top = 1
    {  // Input pad left = 0
      {  // Input pad bottom = 0
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 0, 0, 0, 0>,
            Conv::template process_tile<1, 0, 0, 0, 0, 1>,
            Conv::template process_tile<1, 0, 0, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 0, 0, 1, 0>,
            Conv::template process_tile<1, 0, 0, 0, 1, 1>,
            Conv::template process_tile<1, 0, 0, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 0, 0, 2, 0>,
            Conv::template process_tile<1, 0, 0, 0, 2, 1>,
            Conv::template process_tile<1, 0, 0, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 0, 1, 0, 0>,
            Conv::template process_tile<1, 0, 0, 1, 0, 1>,
            Conv::template process_tile<1, 0, 0, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 0, 1, 1, 0>,
            Conv::template process_tile<1, 0, 0, 1, 1, 1>,
            Conv::template process_tile<1, 0, 0, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 0, 1, 2, 0>,
            Conv::template process_tile<1, 0, 0, 1, 2, 1>,
            Conv::template process_tile<1, 0, 0, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 0, 2, 0, 0>,
            Conv::template process_tile<1, 0, 0, 2, 0, 1>,
            Conv::template process_tile<1, 0, 0, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 0, 2, 1, 0>,
            Conv::template process_tile<1, 0, 0, 2, 1, 1>,
            Conv::template process_tile<1, 0, 0, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 0, 2, 2, 0>,
            Conv::template process_tile<1, 0, 0, 2, 2, 1>,
            Conv::template process_tile<1, 0, 0, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 0, 3, 0, 0>,
            Conv::template process_tile<1, 0, 0, 3, 0, 1>,
            Conv::template process_tile<1, 0, 0, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 0, 3, 1, 0>,
            Conv::template process_tile<1, 0, 0, 3, 1, 1>,
            Conv::template process_tile<1, 0, 0, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 0, 3, 2, 0>,
            Conv::template process_tile<1, 0, 0, 3, 2, 1>,
            Conv::template process_tile<1, 0, 0, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 0
      {  // Input pad bottom = 1
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 1, 0, 0, 0>,
            Conv::template process_tile<1, 0, 1, 0, 0, 1>,
            Conv::template process_tile<1, 0, 1, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 1, 0, 1, 0>,
            Conv::template process_tile<1, 0, 1, 0, 1, 1>,
            Conv::template process_tile<1, 0, 1, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 1, 0, 2, 0>,
            Conv::template process_tile<1, 0, 1, 0, 2, 1>,
            Conv::template process_tile<1, 0, 1, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 1, 1, 0, 0>,
            Conv::template process_tile<1, 0, 1, 1, 0, 1>,
            Conv::template process_tile<1, 0, 1, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 1, 1, 1, 0>,
            Conv::template process_tile<1, 0, 1, 1, 1, 1>,
            Conv::template process_tile<1, 0, 1, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 1, 1, 2, 0>,
            Conv::template process_tile<1, 0, 1, 1, 2, 1>,
            Conv::template process_tile<1, 0, 1, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 1, 2, 0, 0>,
            Conv::template process_tile<1, 0, 1, 2, 0, 1>,
            Conv::template process_tile<1, 0, 1, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 1, 2, 1, 0>,
            Conv::template process_tile<1, 0, 1, 2, 1, 1>,
            Conv::template process_tile<1, 0, 1, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 1, 2, 2, 0>,
            Conv::template process_tile<1, 0, 1, 2, 2, 1>,
            Conv::template process_tile<1, 0, 1, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 1, 3, 0, 0>,
            Conv::template process_tile<1, 0, 1, 3, 0, 1>,
            Conv::template process_tile<1, 0, 1, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 1, 3, 1, 0>,
            Conv::template process_tile<1, 0, 1, 3, 1, 1>,
            Conv::template process_tile<1, 0, 1, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 1, 3, 2, 0>,
            Conv::template process_tile<1, 0, 1, 3, 2, 1>,
            Conv::template process_tile<1, 0, 1, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 1
      {  // Input pad bottom = 2
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 2, 0, 0, 0>,
            Conv::template process_tile<1, 0, 2, 0, 0, 1>,
            Conv::template process_tile<1, 0, 2, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 2, 0, 1, 0>,
            Conv::template process_tile<1, 0, 2, 0, 1, 1>,
            Conv::template process_tile<1, 0, 2, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 2, 0, 2, 0>,
            Conv::template process_tile<1, 0, 2, 0, 2, 1>,
            Conv::template process_tile<1, 0, 2, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 2, 1, 0, 0>,
            Conv::template process_tile<1, 0, 2, 1, 0, 1>,
            Conv::template process_tile<1, 0, 2, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 2, 1, 1, 0>,
            Conv::template process_tile<1, 0, 2, 1, 1, 1>,
            Conv::template process_tile<1, 0, 2, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 2, 1, 2, 0>,
            Conv::template process_tile<1, 0, 2, 1, 2, 1>,
            Conv::template process_tile<1, 0, 2, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 2, 2, 0, 0>,
            Conv::template process_tile<1, 0, 2, 2, 0, 1>,
            Conv::template process_tile<1, 0, 2, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 2, 2, 1, 0>,
            Conv::template process_tile<1, 0, 2, 2, 1, 1>,
            Conv::template process_tile<1, 0, 2, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 2, 2, 2, 0>,
            Conv::template process_tile<1, 0, 2, 2, 2, 1>,
            Conv::template process_tile<1, 0, 2, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 2, 3, 0, 0>,
            Conv::template process_tile<1, 0, 2, 3, 0, 1>,
            Conv::template process_tile<1, 0, 2, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 2, 3, 1, 0>,
            Conv::template process_tile<1, 0, 2, 3, 1, 1>,
            Conv::template process_tile<1, 0, 2, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 2, 3, 2, 0>,
            Conv::template process_tile<1, 0, 2, 3, 2, 1>,
            Conv::template process_tile<1, 0, 2, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 2
      {  // Input pad bottom = 3
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 3, 0, 0, 0>,
            Conv::template process_tile<1, 0, 3, 0, 0, 1>,
            Conv::template process_tile<1, 0, 3, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 3, 0, 1, 0>,
            Conv::template process_tile<1, 0, 3, 0, 1, 1>,
            Conv::template process_tile<1, 0, 3, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 3, 0, 2, 0>,
            Conv::template process_tile<1, 0, 3, 0, 2, 1>,
            Conv::template process_tile<1, 0, 3, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 3, 1, 0, 0>,
            Conv::template process_tile<1, 0, 3, 1, 0, 1>,
            Conv::template process_tile<1, 0, 3, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 3, 1, 1, 0>,
            Conv::template process_tile<1, 0, 3, 1, 1, 1>,
            Conv::template process_tile<1, 0, 3, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 3, 1, 2, 0>,
            Conv::template process_tile<1, 0, 3, 1, 2, 1>,
            Conv::template process_tile<1, 0, 3, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 3, 2, 0, 0>,
            Conv::template process_tile<1, 0, 3, 2, 0, 1>,
            Conv::template process_tile<1, 0, 3, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 3, 2, 1, 0>,
            Conv::template process_tile<1, 0, 3, 2, 1, 1>,
            Conv::template process_tile<1, 0, 3, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 3, 2, 2, 0>,
            Conv::template process_tile<1, 0, 3, 2, 2, 1>,
            Conv::template process_tile<1, 0, 3, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 0, 3, 3, 0, 0>,
            Conv::template process_tile<1, 0, 3, 3, 0, 1>,
            Conv::template process_tile<1, 0, 3, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 0, 3, 3, 1, 0>,
            Conv::template process_tile<1, 0, 3, 3, 1, 1>,
            Conv::template process_tile<1, 0, 3, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 0, 3, 3, 2, 0>,
            Conv::template process_tile<1, 0, 3, 3, 2, 1>,
            Conv::template process_tile<1, 0, 3, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 3
    },  // Input pad left = 0
    {  // Input pad left = 1
      {  // Input pad bottom = 0
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 0, 0, 0, 0>,
            Conv::template process_tile<1, 1, 0, 0, 0, 1>,
            Conv::template process_tile<1, 1, 0, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 0, 0, 1, 0>,
            Conv::template process_tile<1, 1, 0, 0, 1, 1>,
            Conv::template process_tile<1, 1, 0, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 0, 0, 2, 0>,
            Conv::template process_tile<1, 1, 0, 0, 2, 1>,
            Conv::template process_tile<1, 1, 0, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 0, 1, 0, 0>,
            Conv::template process_tile<1, 1, 0, 1, 0, 1>,
            Conv::template process_tile<1, 1, 0, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 0, 1, 1, 0>,
            Conv::template process_tile<1, 1, 0, 1, 1, 1>,
            Conv::template process_tile<1, 1, 0, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 0, 1, 2, 0>,
            Conv::template process_tile<1, 1, 0, 1, 2, 1>,
            Conv::template process_tile<1, 1, 0, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 0, 2, 0, 0>,
            Conv::template process_tile<1, 1, 0, 2, 0, 1>,
            Conv::template process_tile<1, 1, 0, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 0, 2, 1, 0>,
            Conv::template process_tile<1, 1, 0, 2, 1, 1>,
            Conv::template process_tile<1, 1, 0, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 0, 2, 2, 0>,
            Conv::template process_tile<1, 1, 0, 2, 2, 1>,
            Conv::template process_tile<1, 1, 0, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 0, 3, 0, 0>,
            Conv::template process_tile<1, 1, 0, 3, 0, 1>,
            Conv::template process_tile<1, 1, 0, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 0, 3, 1, 0>,
            Conv::template process_tile<1, 1, 0, 3, 1, 1>,
            Conv::template process_tile<1, 1, 0, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 0, 3, 2, 0>,
            Conv::template process_tile<1, 1, 0, 3, 2, 1>,
            Conv::template process_tile<1, 1, 0, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 0
      {  // Input pad bottom = 1
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 1, 0, 0, 0>,
            Conv::template process_tile<1, 1, 1, 0, 0, 1>,
            Conv::template process_tile<1, 1, 1, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 1, 0, 1, 0>,
            Conv::template process_tile<1, 1, 1, 0, 1, 1>,
            Conv::template process_tile<1, 1, 1, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 1, 0, 2, 0>,
            Conv::template process_tile<1, 1, 1, 0, 2, 1>,
            Conv::template process_tile<1, 1, 1, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 1, 1, 0, 0>,
            Conv::template process_tile<1, 1, 1, 1, 0, 1>,
            Conv::template process_tile<1, 1, 1, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 1, 1, 1, 0>,
            Conv::template process_tile<1, 1, 1, 1, 1, 1>,
            Conv::template process_tile<1, 1, 1, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 1, 1, 2, 0>,
            Conv::template process_tile<1, 1, 1, 1, 2, 1>,
            Conv::template process_tile<1, 1, 1, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 1, 2, 0, 0>,
            Conv::template process_tile<1, 1, 1, 2, 0, 1>,
            Conv::template process_tile<1, 1, 1, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 1, 2, 1, 0>,
            Conv::template process_tile<1, 1, 1, 2, 1, 1>,
            Conv::template process_tile<1, 1, 1, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 1, 2, 2, 0>,
            Conv::template process_tile<1, 1, 1, 2, 2, 1>,
            Conv::template process_tile<1, 1, 1, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 1, 3, 0, 0>,
            Conv::template process_tile<1, 1, 1, 3, 0, 1>,
            Conv::template process_tile<1, 1, 1, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 1, 3, 1, 0>,
            Conv::template process_tile<1, 1, 1, 3, 1, 1>,
            Conv::template process_tile<1, 1, 1, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 1, 3, 2, 0>,
            Conv::template process_tile<1, 1, 1, 3, 2, 1>,
            Conv::template process_tile<1, 1, 1, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 1
      {  // Input pad bottom = 2
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 2, 0, 0, 0>,
            Conv::template process_tile<1, 1, 2, 0, 0, 1>,
            Conv::template process_tile<1, 1, 2, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 2, 0, 1, 0>,
            Conv::template process_tile<1, 1, 2, 0, 1, 1>,
            Conv::template process_tile<1, 1, 2, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 2, 0, 2, 0>,
            Conv::template process_tile<1, 1, 2, 0, 2, 1>,
            Conv::template process_tile<1, 1, 2, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 2, 1, 0, 0>,
            Conv::template process_tile<1, 1, 2, 1, 0, 1>,
            Conv::template process_tile<1, 1, 2, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 2, 1, 1, 0>,
            Conv::template process_tile<1, 1, 2, 1, 1, 1>,
            Conv::template process_tile<1, 1, 2, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 2, 1, 2, 0>,
            Conv::template process_tile<1, 1, 2, 1, 2, 1>,
            Conv::template process_tile<1, 1, 2, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 2, 2, 0, 0>,
            Conv::template process_tile<1, 1, 2, 2, 0, 1>,
            Conv::template process_tile<1, 1, 2, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 2, 2, 1, 0>,
            Conv::template process_tile<1, 1, 2, 2, 1, 1>,
            Conv::template process_tile<1, 1, 2, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 2, 2, 2, 0>,
            Conv::template process_tile<1, 1, 2, 2, 2, 1>,
            Conv::template process_tile<1, 1, 2, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 2, 3, 0, 0>,
            Conv::template process_tile<1, 1, 2, 3, 0, 1>,
            Conv::template process_tile<1, 1, 2, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 2, 3, 1, 0>,
            Conv::template process_tile<1, 1, 2, 3, 1, 1>,
            Conv::template process_tile<1, 1, 2, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 2, 3, 2, 0>,
            Conv::template process_tile<1, 1, 2, 3, 2, 1>,
            Conv::template process_tile<1, 1, 2, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 2
      {  // Input pad bottom = 3
        {  // Input pad right = 0
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 3, 0, 0, 0>,
            Conv::template process_tile<1, 1, 3, 0, 0, 1>,
            Conv::template process_tile<1, 1, 3, 0, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 3, 0, 1, 0>,
            Conv::template process_tile<1, 1, 3, 0, 1, 1>,
            Conv::template process_tile<1, 1, 3, 0, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 3, 0, 2, 0>,
            Conv::template process_tile<1, 1, 3, 0, 2, 1>,
            Conv::template process_tile<1, 1, 3, 0, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 0
        {  // Input pad right = 1
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 3, 1, 0, 0>,
            Conv::template process_tile<1, 1, 3, 1, 0, 1>,
            Conv::template process_tile<1, 1, 3, 1, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 3, 1, 1, 0>,
            Conv::template process_tile<1, 1, 3, 1, 1, 1>,
            Conv::template process_tile<1, 1, 3, 1, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 3, 1, 2, 0>,
            Conv::template process_tile<1, 1, 3, 1, 2, 1>,
            Conv::template process_tile<1, 1, 3, 1, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 1
        {  // Input pad right = 2
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 3, 2, 0, 0>,
            Conv::template process_tile<1, 1, 3, 2, 0, 1>,
            Conv::template process_tile<1, 1, 3, 2, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 3, 2, 1, 0>,
            Conv::template process_tile<1, 1, 3, 2, 1, 1>,
            Conv::template process_tile<1, 1, 3, 2, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 3, 2, 2, 0>,
            Conv::template process_tile<1, 1, 3, 2, 2, 1>,
            Conv::template process_tile<1, 1, 3, 2, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 2
        {  // Input pad right = 3
          {  // Output pad bottom = 0
            Conv::template process_tile<1, 1, 3, 3, 0, 0>,
            Conv::template process_tile<1, 1, 3, 3, 0, 1>,
            Conv::template process_tile<1, 1, 3, 3, 0, 2>,
          },  // Output pad bottom = 0
          {  // Output pad bottom = 1
            Conv::template process_tile<1, 1, 3, 3, 1, 0>,
            Conv::template process_tile<1, 1, 3, 3, 1, 1>,
            Conv::template process_tile<1, 1, 3, 3, 1, 2>,
          },  // Output pad bottom = 1
          {  // Output pad bottom = 2
            Conv::template process_tile<1, 1, 3, 3, 2, 0>,
            Conv::template process_tile<1, 1, 3, 3, 2, 1>,
            Conv::template process_tile<1, 1, 3, 3, 2, 2>,
          },  // Output pad bottom = 2
        },  // Input pad right = 3
      },  // Input pad bottom = 3
    },  // Input pad left = 1
  },  // Input pad top = 1
};


template class DepthwiseConvolution<3, 3, 3, 3, 1, 1, float, float>;
}  // namespace depthwise
